#include "fdbuf.h"
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

///////////////////////////////////////////////////////////////////////////////
// Class fdbuf
///////////////////////////////////////////////////////////////////////////////
fdbuf::fdbuf(int fdesc, bool dc, unsigned bufsize)
  : buf(new char[bufsize]),
    length(0),
    start(0),
    errnum(0),
    flags(0),
    size(bufsize),
    fd(fdesc),
    do_close(dc)
{
  if(!buf) {
    flags = flag_error;
    errnum = errno;
  }
  if(fdesc < 0)
    flags |= flag_closed;
}

fdbuf::~fdbuf()
{
  close();
}

bool fdbuf::error() const
{
  return flags & flag_error;
}

bool fdbuf::closed() const
{
  return flags & flag_closed;
}

bool fdbuf::close()
{
  if(do_close && fd >= 0)
    if(::close(fd) == -1) {
      errnum = errno;
      flags |= flag_error;
      return false;
    }
  flags |= flag_closed;
  return true;
}
